@@ -1,11 +1,11 @@
-/*  $Id: init.pl,v 1.150 2007/06/21 14:07:06 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
     E-mail:        wielemak@science.uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2006, University of Amsterdam
+    Copyright (C): 1985-2008, University of Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -76,6 +76,7 @@ discontiguous(Spec)	 :- '$set_pattr'(Spec, (discontiguous)).
 volatile(Spec)		 :- '$set_pattr'(Spec, (volatile)).
 thread_local(Spec)	 :- '$set_pattr'(Spec, (thread_local)).
 noprofile(Spec)		 :- '$set_pattr'(Spec, (noprofile)).
+'$iso'(Spec)		 :- '$set_pattr'(Spec, (iso)).
 
 :- module_transparent
 	'$set_pattr'/2,
@@ -86,7 +87,8 @@ noprofile(Spec)		 :- '$set_pattr'(Spec, (noprofile)).
 	(volatile)/1,
 	(thread_local)/1,
 	(noprofile)/1,
-	'$hide'/2.
+	'$hide'/2,
+	'$iso'/1.
 
 
 		/********************************
@@ -108,7 +110,10 @@ noprofile(Spec)		 :- '$set_pattr'(Spec, (noprofile)).
 	      catch/3,
 	      once/1,
 	      ignore/1,
+	      call_cleanup/2,
 	      call_cleanup/3,
+	      setup_and_call_cleanup/3,
+	      setup_and_call_cleanup/4,
 	      (^)/2)).
 
 :- module_transparent
@@ -135,10 +140,14 @@ noprofile(Spec)		 :- '$set_pattr'(Spec, (noprofile)).
 	ignore/1,
 	block/3,
 	catch/3,
+	setup_and_call_cleanup/3,
+	setup_and_call_cleanup/4,
 	call_cleanup/2,
 	call_cleanup/3,
 	apply/2.
 
+:- '$iso'((call/1, (\+)/1, once/1, (;)/2, (,)/2, (->)/2, catch/3)).
+
 %   ->/2, ;/2, |/2 and \+/1 are normally compiled. These predicate catch them
 %   in case they are called via the meta-call predicates.
 
@@ -157,6 +166,10 @@ noprofile(Spec)		 :- '$set_pattr'(Spec, (noprofile)).
 	Goal1,
 	Goal2.
 
+:- '$iso'((call/2,
+	   call/3,
+	   call/4)).
+
 call(Goal) :-				% make these available as predicates
 	Goal.
 call(G, A) :-
@@ -223,29 +236,34 @@ fail(Label) :-
 	'$cut'(Label),				% handled by compiler
 	fail.
 
-%	catch(:Goal, +Catcher, :Recover)
-%	throw(+Exception)
+%%	catch(:Goal, +Catcher, :Recover)
 %
-%	ISO compliant exception handling.  '$throw'/1 is compiled to
-%	the virtual instruction B_THROW.  See pl-wam.c for details.
+%	ISO compliant exception handling.
 
 catch(_Goal, _Catcher, _Recover) :-
 	'$catch'.
 
-throw(Exception) :-
-	'$throw'(Exception).
-
-%	call_cleanup(:Goal, +Catcher, :Cleanup)
+%%	setup_and_call_cleanup(:Setup, :Goal, :Cleanup).
+%%	setup_and_call_cleanup(:Setup, :Goal, +Catcher, :Cleanup).
+%%	call_cleanup(:Goal, :Cleanup)
+%%	call_cleanup(:Goal, +Catcher, :Cleanup)
 %
 %	Call Cleanup as Goal finished (deterministic success, failure,
 %	exception or cut).  '$call_cleanup' translated to I_CALLCLEANUP.
 
-call_cleanup(Goal, Cleanup) :-
-	call_cleanup(Goal, _Catcher, Cleanup).
-
-call_cleanup(_Goal, _Catcher, _Cleanup) :-
+setup_and_call_cleanup(Setup, _Goal, _Catcher, _Cleanup) :-
+	Setup,
 	'$call_cleanup'.
 
+setup_and_call_cleanup(Setup, Goal, Cleanup) :-
+	setup_and_call_cleanup(Setup, Goal, _Catcher, Cleanup).
+
+call_cleanup(Goal, Cleanup) :-
+	setup_and_call_cleanup(true, Goal, _Catcher, Cleanup).
+
+call_cleanup(Goal, Catcher, Cleanup) :-
+	setup_and_call_cleanup(true, Goal, Catcher, Cleanup).
+
 
 		/********************************
 		*            MODULES            *
@@ -381,11 +399,17 @@ user:file_search_path(foreign, swi(ArchLib)) :-
 	atom_concat('lib/', Arch, ArchLib).
 user:file_search_path(foreign, swi(lib)).
 user:file_search_path(user_profile, '.').
-user:file_search_path(user_profile, UserHome) :-
+user:file_search_path(user_profile, app_preferences('.')).
+user:file_search_path(app_preferences, PrologAppData) :-
+	current_prolog_flag(windows, true),
+	catch(win_folder(appdata, AppData), _, fail),
+	atom_concat(AppData, '/SWI-Prolog', PrologAppData),
+	(   exists_directory(PrologAppData)
+	->  true
+	;   catch(make_directory(PrologAppData), _, fail)
+	).
+user:file_search_path(app_preferences, UserHome) :-
 	catch(expand_file_name(~, [UserHome]), _, fail).
-user:file_search_path(user_profile, SwiHome) :-
-	current_prolog_flag(windows, true), 		% single user machine
-	current_prolog_flag(home, SwiHome).
 
 
 %	expand_file_search_path(+Spec, -Expanded)
@@ -809,6 +833,8 @@ preprocessor(Old, New) :-
 	consult/1,
 	use_module/1,
 	use_module/2,
+	reexport/1,
+	reexport/2,
 	'$load_file'/3,
 	load_files/1,
 	load_files/2.
@@ -839,12 +865,34 @@ use_module(Files) :-
 %	As use_module/1, but takes only one file argument and imports only
 %	the specified predicates rather than all public predicates.
 
-use_module(Files, Import) :-
+use_module(File, Import) :-
+	load_files(File, [ if(not_loaded),
+			   must_be_module(true),
+			   imports(Import)
+			 ]).
+
+%%	reexport(+Files)
+%
+%	As use_module/1, exporting all imported predicates.
+
+reexport(Files) :-
 	load_files(Files, [ if(not_loaded),
 			    must_be_module(true),
-			    imports(Import)
+			    reexport(true)
 			  ]).
 
+%%	reexport(+File, +ImportList)
+%
+%	As use_module/1, re-exporting all imported predicates.
+
+reexport(File, Import) :-
+	load_files(File, [ if(not_loaded),
+			   must_be_module(true),
+			   imports(Import),
+			   reexport(true)
+			 ]).
+
+
 [X] :- !,
 	consult(X).
 [F|R] :-
@@ -896,7 +944,7 @@ load_files(Files, Options) :-
 	).
 '$load_files'(X, _, _) :-
 	var(X), !,
-	throw(error(instantiation_error, _)).
+	throw(error(instantiation_error, context(load_files/2,_))).
 '$load_files'([], _, _) :- !.
 '$load_files'([H|T], Module, Options) :- !,
 	'$load_files'(H, Module, Options),
@@ -928,7 +976,7 @@ load_files(Files, Options) :-
 	time_file(Derived, Modified),
         Modified @=< LoadTime, !.
 '$noload'(changed, FullFile) :-
-	'$time_source_file'(FullFile, LoadTime),
+	'$time_source_file'(FullFile, LoadTime, user),
         time_file(FullFile, Modified),
         Modified @=< LoadTime, !.
 
@@ -979,6 +1027,7 @@ load_files(Files, Options) :-
 	),
 	    
 	'$get_option'(imports(Import), Options, all),
+	'$get_option'(reexport(Reexport), Options, false),
 	'$get_option'(must_be_module(IsModule), Options, false),
 	current_prolog_flag(verbose_load, DefVerbose),
 	'$negate'(DefVerbose, DefSilent),
@@ -999,7 +1048,7 @@ load_files(Files, Options) :-
 	(   var(FromStream),
 	    '$noload'(If, FullFile)
 	->  (   '$current_module'(LoadModule, FullFile)
-	    ->  '$import_list'(Module, LoadModule, Import)
+	    ->  '$import_list'(Module, LoadModule, Import, Reexport)
 	    ;   (   Module == user
 		->  true
 		;   '$load_file'(File, Module, [if(true)|Options])
@@ -1020,22 +1069,27 @@ load_files(Files, Options) :-
 	    ),
 
 	    '$print_message'(silent /*MessageLevel*/,
-			   load_file(start(Level,
-					   file(File, Absolute)))),
+			     load_file(start(Level,
+					     file(File, Absolute)))),
 	    (   nonvar(FromStream),
 	        '$consult_file'(stream(Absolute, FromStream), Encoding,
-			      Module, Import, IsModule, Action, LM)
+				Module, IsModule, Action, LM)
 	    ->	true
 	    ;   var(FromStream),
 		'$consult_goal'(Absolute, Goal),
 		call(Goal,
 		     Absolute, Encoding,
-		     Module, Import, IsModule, Action, LM)
+		     Module, IsModule, Action, LM)
 	    ->  true
 	    ;   print_message(error, load_file(failed(File))),
 		fail
 	    ),
 
+	    (	atom(LM)
+	    ->  '$import_list'(Module, LM, Import, Reexport)
+	    ;	true
+	    ),
+
 	    (	Level == 0
 	    ->	garbage_collect_clauses
 	    ;	true
@@ -1076,36 +1130,64 @@ load_files(Files, Options) :-
 	print_message(error, E),
 	fail.
 
-%	'$consult_file'(+Path, +Encoding,
-%		      +Module, +Import, +IsModule, -Action, -LoadedIn)
+%%	'$consult_file'(+Path, +Encoding, +Module, +IsModule, -Action, -LoadedIn)
 
-'$consult_file'(Absolute, Enc, Module, Import, IsModule, What, LM) :-
+'$consult_file'(Absolute, Enc, Module, IsModule, What, LM) :-
 	'$set_source_module'(Module, Module), !, % same module
-	'$consult_file_2'(Absolute, Enc, Module, Import, IsModule, What, LM).
-'$consult_file'(Absolute, Enc, Module, Import, IsModule, What, LM) :-
+	'$consult_file_2'(Absolute, Enc, Module, IsModule, What, LM).
+'$consult_file'(Absolute, Enc, Module, IsModule, What, LM) :-
 	'$set_source_module'(OldModule, Module),
 	'$ifcompiling'('$qlf_start_sub_module'(Module)),
-        '$consult_file_2'(Absolute, Enc, Module, Import, IsModule, What, LM),
+        '$consult_file_2'(Absolute, Enc, Module, IsModule, What, LM),
 	'$ifcompiling'('$qlf_end_part'),
 	'$set_source_module'(_, OldModule).
 
-'$consult_file_2'(Absolute, Enc, Module, Import, IsModule, What, LM) :-
+'$consult_file_2'(Absolute, Enc, Module, IsModule, What, LM) :-
 	'$set_source_module'(OldModule, Module),	% Inform C we start loading
 	'$load_id'(Absolute, Id),
 	'$start_consult'(Id),
 	'$compile_type'(What),
 	(   flag('$compiling', wic, wic)	% TBD
-	->  '$add_directive_wic'('$assert_load_context_module'(Id,OldModule))
+	->  '$add_directive_wic'('$assert_load_context_module'(Id, OldModule))
 	;   true
 	),
 	'$assert_load_context_module'(Id, OldModule),
 
-	'$style_check'(OldStyle, OldStyle),	% Save style parameters
+	
+	'$save_lex_state'(LexState),
 	'$open_source'(Absolute, Enc, In,
-		     '$load_file'(In, Id, Import, IsModule, LM)),
-	'$style_check'(_, OldStyle),		% Restore old style
+		     '$load_file'(In, Id, IsModule, LM)),
+	'$restore_lex_state'(LexState),
 	'$set_source_module'(_, OldModule).	% Restore old module
 
+:- thread_local
+	'$current_load_dialect'/1.
+
+%%	'$set_dialect'(-Old, +New) is det.
+%
+%	Switch the currently expected dialect.
+
+'$set_dialect'(Old, New) :-
+	Old == New, !,
+	(   '$current_load_dialect'(Dialect)
+	->  true
+	;   Dialect = swi
+	).
+'$set_dialect'(Old, Dialect) :-
+	(   retract('$current_load_dialect'(Old))
+	->  true
+	;   Old = swi
+	),
+	assert('$current_load_dialect'(Dialect)).
+
+'$save_lex_state'(lexstate(Style, Dialect)) :-
+	'$style_check'(Style, Style),
+	'$set_dialect'(Dialect, Dialect).
+'$restore_lex_state'(lexstate(Style, Dialect)) :-
+	'$style_check'(_, Style),
+	'$set_dialect'(_, Dialect).
+
+
 '$load_id'(stream(Id, _), Id) :- !.
 '$load_id'(Id, Id).
 
@@ -1118,43 +1200,50 @@ load_files(Files, Options) :-
 	;   What = 'boot compiled'
 	).
 
-%	'$load_context_module'(+File, -Module)
+%%	'$load_context_module'(+File, -Module)
+%	
 %	Record the module a file was loaded from (see make/0)
+%	
+%	@tbd	Should also know which predicates are imported!
 
-'$load_context_module'(File, Module) :-
-	recorded('$load_context_module', File/Module, _).
+:- dynamic
+	'$load_context_module'/2.
 
 '$assert_load_context_module'(File, Module) :-
-	recorded('$load_context_module', File/Module, _), !.
-'$assert_load_context_module'(File, Module) :-
-	recordz('$load_context_module', File/Module, _).
+	(   '$load_context_module'(File, Module)
+	->  true
+	;   assert('$load_context_module'(File, Module))
+	).
 
-%   '$load_file'(+FirstTerm, +In, +Path, +Import, +IsModule, -Module)
+
+%   '$load_file'(+In, +Path, +IsModule, -Module)
 %
-%   '$load_file'/6 does the actual loading. The first term has already been
-%   read as this may be the module declaraction.
+%   '$load_file'/4 does the actual loading.
 
-'$load_file'(In, File, Import, IsModule, Module) :-
+'$load_file'(In, File, IsModule, Module) :-
 	(   peek_char(In, #)
 	->  skip(In, 10)
 	;   true
 	),
 	'$read_clause'(In, First),
-	'$load_file'(First, In, File, Import, IsModule, Module).
+	'$load_file'(First, In, File, IsModule, Module).
 
-'$load_file'((?- module(Module, Public)), In, File, all, _, Module) :- !,
-	'$load_module'(Module, Public, all, In, File).
-'$load_file'((:- module(Module, Public)), In, File, all, _, Module) :- !,
-	'$load_module'(Module, Public, all, In, File).
-'$load_file'((?- module(Module, Public)), In, File, Import, _, Module) :- !,
-	'$load_module'(Module, Public, Import, In, File).
-'$load_file'((:- module(Module, Public)), In, File, Import, _, Module) :- !,
-	'$load_module'(Module, Public, Import, In, File).
-'$load_file'(_, _, File, _, true, _) :- !,
+
+'$load_file'((?- module(Module, Public)), In, File, _, Module) :- !,
+	'$load_module'(Module, Public, In, File).
+'$load_file'((:- module(Module, Public)), In, File, _, Module) :- !,
+	'$load_module'(Module, Public, In, File).
+'$load_file'((?- module(Module, Public)),
+	     In, File, _, Module) :- !,
+	'$load_module'(Module, Public, In, File).
+'$load_file'((:- module(Module, Public)),
+	     In, File, _, Module) :- !,
+	'$load_module'(Module, Public, In, File).
+'$load_file'(_, _, File, true, _) :- !,
 	throw(error(domain_error(module_file, File), _)).
-'$load_file'(end_of_file, _, _, _, _, Module) :- !, 	% empty file
+'$load_file'(end_of_file, _, _, _, Module) :- !, 	% empty file
 	'$set_source_module'(Module, Module).
-'$load_file'(FirstClause, In, File, _, false, Module) :- !,
+'$load_file'(FirstClause, In, File, false, Module) :- !,
 	'$set_source_module'(Module, Module),
 	'$ifcompiling'('$qlf_start_file'(File)),
 	ignore('$consult_clause'(FirstClause, File)),
@@ -1164,42 +1253,113 @@ load_files(Files, Options) :-
 '$reserved_module'(system).
 '$reserved_module'(user).
 
-'$load_module'(Reserved, _, _, _, _) :-
+'$load_module'(Reserved, _, _, _) :-
 	'$reserved_module'(Reserved), !,
 	throw(error(permission_error(load, module, Reserved), _)).
-'$load_module'(Module, Public, Import, In, File) :-
+'$load_module'(Module, Public, In, File) :-
 	'$set_source_module'(OldModule, OldModule),
 	source_location(_File, Line),
 	'$declare_module'(Module, File, Line),
-	'$export_list'(Public, Module, File, Ops),
+	'$export_list'(Public, Module, Ops),
 	'$ifcompiling'('$qlf_start_module'(Module)),
 	'$export_ops'(Ops, Module, File),
 	'$consult_stream'(In, File),
 	Module:'$check_export',
-	'$ifcompiling'('$qlf_end_part'),
-	'$import_list'(OldModule, Module, Import).
+	'$ifcompiling'('$qlf_end_part').
 
 
-'$import_list'(_, _, []) :- !.
-'$import_list'(Module, Source, [Name/Arity|Rest]) :- !,
-	functor(Term, Name, Arity),
-	'$import_wic'(Source, Term),
-	ignore(Module:import(Source:Term)),
-	'$import_list'(Module, Source, Rest).
-'$import_list'(Context, Module, all) :- !,
-	export_list(Module, Exports),
-	'$import_all'(Exports, Context, Module),
-	'$import_ops'(Context, Module).
+%%	'$import_list'(+TargetModule, +FromModule, +Import, +Reexport) is det.
+%
+%	Import from FromModule to TargetModule. Import  is one of =all=,
+%	a list of optionally  mapped  predicate   indicators  or  a term
+%	except(Import).
+
+'$import_list'(_, _, Var, _) :-
+	var(Var), !,
+	throw(error(instantitation_error, _)).
+'$import_list'(Target, Source, all, Reexport) :- !,
+	export_list(Source, Import),
+	'$import_ops'(Target, Source),
+	'$import_list'(Target, Source, Import, Reexport).
+'$import_list'(Target, Source, except(Spec), Reexport) :- !,
+	export_list(Source, Export),
+	(   is_list(Spec)
+	->  true
+	;   throw(error(type_error(list, Spec), _))
+	),
+	'$import_except'(Spec, Export, Import),
+	'$import_list'(Target, Source, Import, Reexport).
+'$import_list'(Target, Source, Import, Reexport) :- !,
+	is_list(Import), !,
+	'$import_all'(Import, Target, Source, Reexport).
+'$import_list'(_, _, Import, _) :-
+	throw(error(type_error(import_specifier, Import))).
 
 
-'$import_all'([], _, _).
-'$import_all'([Head|Rest], Context, Source) :-
-	ignore(Context:import(Source:Head)),
-	'$import_wic'(Source, Head),
-	'$import_all'(Rest, Context, Source).
+:- op(700, xfx, user:as).		% TBD: kernel table
+
+'$import_except'([], List, List).
+'$import_except'([H as N|T], List0, List) :- !,
+	'$import_as'(H, N, List0, List1),
+	'$import_except'(T, List1, List).
+'$import_except'([H|T], List0, List) :-
+	'$select'(P, List0, List1),
+	'$same_pi'(H, P), !,
+	'$import_except'(T, List1, List).
+
+'$import_as'(PI, N, [PI2|T], [PI as N|T]) :-
+	'$same_pi'(PI, PI2), !.
+'$import_as'(PI, N, [H|T0], [H|T]) :- !,
+	'$import_as'(PI, N, T0, T).
+'$import_as'(PI, _, _, _) :-
+	throw(error(existence_error(export, PI), _)).
+
+'$same_pi'(PI1, PI2) :-
+	'$canonical_pi'(PI1, PI),
+	'$canonical_pi'(PI2, PI).
+
+'$canonical_pi'(N//A0, N/A) :- !,
+	A is A0 + 2.
+'$canonical_pi'(PI, PI).
 
 
-%	'$import_ops'(+Target, +Source)
+'$import_all'(Import, Context, Source, Reexport) :-
+	'$import_all2'(Import, Context, Source, Imported),
+	(   Reexport == true,
+	    '$list_to_conj'(Imported, Conj)
+	->  export(Context:Conj),
+	    (	flag('$compiling', wic, wic)
+	    ->	'$add_directive_wic'(export(Context:Conj))
+	    ;	true
+	    )
+	;   true
+	).
+
+'$import_all2'([], _, _, []).
+'$import_all2'([PI as NewName|Rest], Context, Source,
+	       [NewName/Arity|Imported]) :- !,
+	'$canonical_pi'(PI, Name/Arity),
+	length(Args, Arity),
+	Head =.. [Name|Args],
+	NewHead =.. [NewName|Args],
+	source_location(File, _Line),
+	(   '$get_predicate_attribute'(Source:Head, transparent, 1)
+	->  '$set_predicate_attribute'(Context:NewHead, transparent, 1)
+	;   true
+	),
+	'$store_clause'((NewHead :- Source:Head), File),
+	'$import_all2'(Rest, Context, Source, Imported).
+'$import_all2'([Pred|Rest], Context, Source, [Pred|Imported]) :-
+	Context:import(Source:Pred),
+	'$import_wic'(Source, Pred),
+	'$import_all2'(Rest, Context, Source, Imported).
+
+
+'$list_to_conj'([One], One) :- !.
+'$list_to_conj'([H|T], (H,Rest)) :-
+	'$list_to_conj'(T, Rest).
+
+%%	'$import_ops'(+Target, +Source)
 %	
 %	Import the operators export from Source into the module table of
 %	Target.
@@ -1213,46 +1373,41 @@ load_files(Files, Options) :-
 	).
 
 
-%	'$export_list'(+Declarations, +Module, +File, -Ops)
+%%	'$export_list'(+Declarations, +Module, -Ops)
 %	
 %	Handle the export list of the module declaration for Module
 %	associated to File.
 
-'$export_list'(Var, _, _, []) :-
+'$export_list'(Decls, Module, Ops) :-
+	is_list(Decls), !,
+	'$do_export_list'(Decls, Module, Ops).
+'$export_list'(Decls, _, _) :-
+	var(Decls),
+	throw(error(instantiation_error, _)).
+'$export_list'(Decls, _, _) :-
+	throw(error(type_error(list, Decls), _)).
+
+'$do_export_list'([], _, []) :- !.
+'$do_export_list'([H|T], Module, Ops) :- !,
+	'$export1'(H, Module, Ops, Ops1),
+	'$do_export_list'(T, Module, Ops1).
+
+'$export1'(Var, _, _, _) :-
 	var(Var), !,
-	print_message(error, error(type_error(list, Var), _)).
-'$export_list'([H|T], Module, File, Ops) :- !,
-	(   H = op(_,_,_)
-	->  Ops = [H|RestOps]
-	;   catch('$export1'(H, Module, File), E, print_message(error, E))
-	->  RestOps = Ops
-	;   print_message(error, error(type_error(export_declaration, H), _)),
-	    RestOps = Ops
-	),
-	'$export_list'(T, Module, File, RestOps).
-'$export_list'([], _, _, []) :- !.
-'$export_list'(Term, _, _, []) :-
-	print_message(error, error(type_error(list, Term), _)).
+	throw(error(instantiation_error, _)).
+'$export1'(Op, _, [Op|T], T) :-
+	Op = op(_,_,_), !.
+'$export1'(PI, Module, Ops, Ops) :-
+	export(Module:PI).
 
-
-'$export_ops'([H|T], Module, File) :-
-	(   catch('$export1'(H, Module, File), E, print_message(error, E))
-	->  true
-	;   print_message(error, error(type_error(export_declaration, H), _))
-	),
+'$export_ops'([op(Pri, Assoc, Name)|T], Module, File) :-
+	op(Pri, Assoc, Module:Name),
+	'$store_clause'('$exported_op'(Pri, Assoc, Name), File),
 	'$export_ops'(T, Module, File).
 '$export_ops'([], _, _).
 
 
-'$export1'(Name/Arity, Module, _File) :-
-	functor(Term, Name, Arity), !,
-	export(Module:Term).
-'$export1'(op(Pri, Assoc, Name), Module, File) :-
-	op(Pri, Assoc, Module:Name),
-	'$store_clause'('$exported_op'(Pri, Assoc, Name), File).
-
-
-%	'$consult_stream'(+Stream, +File)
+%%	'$consult_stream'(+Stream, +File)
 %
 %	Read and record all clauses until the rest of the file.
 
@@ -1399,6 +1554,9 @@ expand_term(Term, Expanded) :-		% local term-expansion
 expand_term(Head --> Body, Expanded) :-
 	'$translate_rule'(Head --> Body, Expanded0), !,
 	'$expand_clauses'(Expanded0, Expanded).
+expand_term(Term, []) :-
+	'$if_expansion'(Term, X),
+	X == [], !.
 expand_term(Term0, Term) :-
 	'$goal_expansion_module'(_), !,
 	'$expand_clauses'(Term0, Term).
@@ -1430,10 +1588,30 @@ expand_term(Term, Term).
 	'$qlf_assert_clause'(Ref, development).
 
 
+%%	compile_aux_clauses(+Clauses) is det.
+%
+%	Compile clauses given the current  source   location  but do not
+%	change  the  notion  of   the    current   procedure  such  that
+%	discontiguous  warnings  are  not  issued.    The   clauses  are
+%	associated with the current file and  therefore wiped out if the
+%	file is reloaded.
+
+compile_aux_clauses(Clauses) :-
+	source_location(File, _Line),
+	'$start_aux'(File, Context),
+	call_cleanup('$store_clause'(Clauses, File),
+		     '$end_aux'(File, Context)).
+
+
 		 /*******************************
 		 *	     INCLUDE		*
 		 *******************************/
 
+:- multifile
+	'$included'/3.			% Into, File, LastModified
+:- dynamic
+	'$included'/3.
+
 '$expand_include'(File, FileInto) :-
 	absolute_file_name(File,
 			   [ file_type(prolog),
@@ -1441,10 +1619,12 @@ expand_term(Term, Term).
 			   ], Path),
 	'$push_input_context',
 	open(Path, read, In),
+	time_file(Path, Time),
 	'$read_clause'(In, Term0),
 	'$read_include_file'(Term0, In, Terms),
 	close(In),
 	'$pop_input_context',
+	'$store_clause'(system:'$included'(FileInto, Path, Time), FileInto),
 	'$consult_clauses'(Terms, FileInto).
 
 '$read_include_file'(end_of_file, _, []) :- !.
@@ -1458,6 +1638,60 @@ expand_term(Term, Term).
 	'$consult_clauses'(T, File).
 
 
+		 /*******************************
+		 *	:- IF ... :- ENDIF	*
+		 *******************************/
+
+:- thread_local
+	'$include_code'/1.
+
+'$including' :-
+	'$include_code'(X), !,
+	X == true.
+'$including'.
+
+'$if_expansion'((:- if(G)), []) :-
+	(   '$including'
+	->  (   catch('$eval_if'(G), E, (print_message(error, E), fail))
+	    ->  asserta('$include_code'(true))
+	    ;   asserta('$include_code'(false))
+	    )
+	;   asserta('$include_code'(else_false))
+	).
+'$if_expansion'((:- elif(G)), []) :-
+	(   retract('$include_code'(Old))
+	->  (   Old == true
+	    ->  asserta('$include_code'(else_false))
+	    ;   Old == false,
+		catch('$eval_if'(G), E, (print_message(error, E), fail))
+	    ->  asserta('$include_code'(true))
+	    ;	asserta('$include_code'(Old))
+	    )
+	;    throw(error(context_error(no_if), _))
+	).
+'$if_expansion'((:- else), []) :-
+	(   retract('$include_code'(X))
+	->  (   X == true
+	    ->  X2 = false 
+	    ;   X == false
+	    ->	X2 = true
+	    ;	X2 = X
+	    ),
+	    asserta('$include_code'(X2))
+	;   throw(error(context_error(no_if), _))
+	).
+'$if_expansion'(end_of_file, end_of_file) :- !. % TBD: Check completeness
+'$if_expansion'((:- endif), []) :-
+	retract('$include_code'(_)), !.
+
+'$if_expansion'(_, []) :-
+	\+ '$including'.
+
+'$eval_if'(G) :-
+	expand_goal(G, G2),
+	G2.
+
+
 		 /*******************************
 		 *	       READING		*
 		 *******************************/
@@ -1573,6 +1807,8 @@ expand_goal(A, B) :-
         '$do_expand_body'(B, EB).
 '$do_expand_body'(findall(V, G, B), findall(V, EG, B)) :- !,
         '$do_expand_body'(G, EG).
+'$do_expand_body'(findall(V, G, B, T), findall(V, EG, B, T)) :- !,
+        '$do_expand_body'(G, EG).
 '$do_expand_body'(bagof(V, G, B), bagof(V, EG, B)) :- !,
         '$do_expand_body'(G, EG).
 '$do_expand_body'(setof(V, G, B), setof(V, EG, B)) :- !,
@@ -1636,31 +1872,29 @@ The original version contained  '$t_tidy'/2  to   convert  ((a,b),  c)  to
 resulting code is simply the same), I've removed that.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-'$translate_rule'((LP-->List), H) :-
-	is_list(List), !,
-	(   List = []
-	->  '$t_head'(LP, S, S, H)
-        ;   List = [X]
-        ->  '$t_head'(LP, [X|S], S, H)
-        ;   '$append'(List, SR, S),
-            '$extend'(LP, S, SR, H)
-        ).
+'$translate_rule'(((LP,MNT)-->RP),(H:-B)) :- !,
+	( var(LP) -> throw(error(instantiation_error,_)) ; true ),
+	'$extend'(LP, S0, SR, H),
+	'$t_body'(RP, S0, S1, B0),
+	'$t_body'(MNT, SR, S1,M),
+	'$body_optimized'((B0,M),B1,S0),
+	'$body_optimized'(B1,B,SR).
 '$translate_rule'((LP-->RP), (H:-B)):-
-	'$t_head'(LP, S0, SR, H),
-	'$t_body'(RP, S0, SR, B0),
+	'$extend'(LP, S0, S, H),
+	'$t_body'(RP, S0, S, B0),
+	'$body_optimized'(B0,B,S0).
+
+'$body_optimized'(B0,B,S0) :-
 	(   B0 = (S00=X, B),		% map a(H,T) :- H = [a,b|T], b(T)
 	    S00 == S0
-	->  S0 = X			% into a([a,b|T]) :- b(T).
+	->  S0 = X			% into a([a,b|T0]) :- b(T0, T).
+	;   B0 = (S00=X),		% map a(H,T) :- H = [a,b|T]
+	    S00 == S0
+	->  S0 = X,			% into a([a,b|T], T)
+	    B = true
 	;   B0 = B
 	).
 
-'$t_head'((LP, List), S, SR, H) :-
-	'$append'(List, SR, List2), !,
-	'$extend'(LP, S, List2, H).
-'$t_head'(LP, S, SR, H) :-
-	'$extend'(LP, S, SR, H).
-
-
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 On the DCG Translation of {}
 
@@ -1721,11 +1955,18 @@ then the call p([a], [a]) will succeed, which is quite definitely wrong.
 	var(Var), !.
 '$t_body'([], S, SR, S=SR) :- !.		% inline lists
 '$t_body'(List, S, SR, C) :-
-	List = [_|_], !,
-	(   is_list(List)
-	->  '$append'(List, SR, OL),
+	(   List = [_|_]
+	->  !,
+	    (   is_list(List)
+	    ->  '$append'(List, SR, OL),
+		C = (S = OL)
+	    ;   C = '$append'(List, SR, S)	% Deals with [H|T] in body
+	    )
+	;   string(List)
+	->  !,
+	    string_to_list(List, Codes),
+	    '$append'(Codes, SR, OL),
 	    C = (S = OL)
-	;   C = '$append'(List, SR, S)	% Deals with [H|T] in body
 	).
 '$t_body'(!, S, S, !) :- !.
 '$t_body'({}, S, S, true) :- !.
@@ -1767,11 +2008,29 @@ then the call p([a], [a]) will succeed, which is quite definitely wrong.
 :- dynamic  '$extend_cache'/4.
 :- volatile '$extend_cache'/4.
 
+'$dcg_reserved'([]).
+'$dcg_reserved'([_|_]).
+'$dcg_reserved'({_}).
+'$dcg_reserved'({}).
+'$dcg_reserved'(!).
+'$dcg_reserved'((\+_)).
+'$dcg_reserved'((_,_)).
+'$dcg_reserved'((_;_)).
+'$dcg_reserved'((_|_)).
+'$dcg_reserved'((_->_)).
+'$dcg_reserved'((_*->_)).
+'$dcg_reserved'((_-->_)).
+
+'$extend'(V, _, _, _) :-
+	var(V), !,
+	throw(error(instantiation_error,_)).
 '$extend'(M:OldT, A1, A2, M:NewT) :- !,
 	'$extend'(OldT, A1, A2, NewT).
 '$extend'(OldT, A1, A2, NewT) :-
 	'$extend_cache'(OldT, A1, A2, NewT), !.
 '$extend'(OldT, A1, A2, NewT) :-
+	( callable(OldT) -> true ; throw(error(type_error(callable,OldT),_)) ),
+	( '$dcg_reserved'(OldT) -> throw(error(permission_error(define,dcg_nonterminal,OldT),_)) ; true ),
 	functor(OldT, Name, Arity),
 	functor(CopT, Name, Arity),
 	NewArity is Arity+2,
@@ -1797,11 +2056,14 @@ then the call p([a], [a]) will succeed, which is quite definitely wrong.
 :- module_transparent
 	phrase/2,
 	phrase/3.
+:- noprofile((phrase/2,
+	      phrase/3)).
 
 phrase(RuleSet, Input) :-
 	phrase(RuleSet, Input, []).
 phrase(RuleSet, Input, Rest) :-
 	strip_module(RuleSet, M, Plain),
+	( var(Plain) -> throw(error(instantiation_error,_)) ; true ),
 	'$t_body'(Plain, S0, S, Body),
 	Input = S0, Rest = S,
 	M:Body.
@@ -1899,6 +2161,8 @@ saved state.
 		 *	       HALT		*
 		 *******************************/
 
+:- '$iso'((halt/0)).
+
 halt :-
 	halt(0).
 
@@ -1930,11 +2194,12 @@ at_halt(Spec) :-
 
 '$load_wic_files'(Module, Files) :-
 	'$execute_directive'('$set_source_module'(OldM, Module), []),
-	'$style_check'(OldS, 2'1111),
+	'$save_lex_state'(LexState),
+	'$style_check'(_, 2'1111),
 	flag('$compiling', OldC, wic),
 	consult(Files),
 	'$execute_directive'('$set_source_module'(_, OldM), []),
-	'$execute_directive'('$style_check'(_, OldS), []),
+	'$execute_directive'('$restore_lex_state'(LexState), []),
 	flag('$compiling', _, OldC).
 
 
