@@ -1,4 +1,4 @@
-/*  $Id: backcomp.pl,v 1.24 2006/08/29 20:28:01 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
@@ -50,8 +50,15 @@
 	    proper_list/1,
 	    free_variables/2,		% +Term, -Variables
 	    checklist/2,		% :Goal, +List
+	    sublist/3,			% :Goal, +List, -Sublist
 	    convert_time/2,		% +Stamp, -String
-	    convert_time/8		% +String, -YMDmhs.ms
+	    convert_time/8,		% +String, -YMDmhs.ms
+	    'C'/3,			% +List, -Head, -Tail
+	    current_thread/2,		% ?Thread, ?Status
+	    current_mutex/3,		% ?Mutex, ?Owner, ?Count
+	    message_queue_size/2,	% +Queue, -TermsWaiting
+	    lock_predicate/2,		% +Name, +Arity
+	    unlock_predicate/2		% +Name, +Arity
 	  ]).
 
 /** <module> Backward compatibility
@@ -68,7 +75,8 @@ compatibility predicates.
 '$arch'(Arch, unknown) :-
 	current_prolog_flag(arch, Arch).
 '$version'(Version) :-
-	current_prolog_flag(version, Version).
+	current_prolog_flag(version_data, swi(Major, Minor, Patch, _)),
+	Version is 10000*Major+100*Minor+Patch.
 '$home'(Home) :-
 	current_prolog_flag(home, Home).
 '$argv'(Argv) :-
@@ -79,7 +87,7 @@ compatibility predicates.
 %	
 %	Write term ignoring operators and quote atoms.
 %	
-%	@depreciated Use write_term/3 or write_canonical/2.
+%	@deprecated Use write_term/3 or write_canonical/2.
 
 displayq(Term) :-
 	write_term(Term, [ignore_ops(true),quoted(true)]).
@@ -90,18 +98,18 @@ displayq(Stream, Term) :-
 %%	sformat(-String, +Format, +Args) is det.
 %%	sformat(-String, +Format) is det.
 %
-%	@depreciated Use format/3 as =|format(string(String), ...)|=
+%	@deprecated Use format/3 as =|format(string(String), ...)|=
 
 :- module_transparent sformat/2, sformat/3.
 
+sformat(String, Format) :-
+	format(string(String), Format, []).
 sformat(String, Format, Arguments) :-
 	format(string(String), Format, Arguments).
-sformat(String, Format) :-
-	format(string(String), Format).
 
 %%	concat(+Atom1, +Atom2, -Atom) is det.
 %
-%	@depreciated Use ISO atom_concat/3
+%	@deprecated Use ISO atom_concat/3
 
 concat(A, B, C) :-
 	atom_concat(A, B, C).
@@ -109,7 +117,7 @@ concat(A, B, C) :-
 %%	read_variables(-Term, -Bindings) is det.
 %%	read_variables(+In:stream, -Term, -Bindings) is det.
 %
-%	@depreciated Use ISO read_term/[2,3].
+%	@deprecated Use ISO read_term/[2,3].
 
 read_variables(Term, Vars) :-
 	read_term(Term, [variable_names(Vars)]).
@@ -122,7 +130,7 @@ read_variables(Stream, Term, Vars) :-
 %
 %	Control Prolog flags.
 %
-%	@depreciated Use ISO current_prolog_flag/2 and set_prolog_flag/2.
+%	@deprecated Use ISO current_prolog_flag/2 and set_prolog_flag/2.
 
 feature(Key, Value) :-
 	current_prolog_flag(Key, Value).
@@ -134,7 +142,7 @@ set_feature(Key, Value) :-
 %
 %	Predecessor of sub_string using 1-based Offset.
 %	
-%	@depreciated Use sub_string/5.
+%	@deprecated Use sub_string/5.
 
 substring(String, Offset, Length, Sub) :-
 	Offset0 is Offset - 1,
@@ -142,14 +150,14 @@ substring(String, Offset, Length, Sub) :-
 
 %%	flush is det.
 %
-%	@depreciated use ISO flush_output/0.
+%	@deprecated use ISO flush_output/0.
 
 flush :-
 	flush_output.
 
 %%	write_ln(X) is det
 %
-%	@depreciated Use writeln(X).
+%	@deprecated Use writeln(X).
 
 write_ln(X) :-
 	write(X), nl.
@@ -160,7 +168,7 @@ write_ln(X) :-
 %	in a [].  There is not much use for the quick is_list, as in
 %	most cases you want to process the list element-by-element anyway.
 %	
-%	@depreciated Use ISO is_list/1.
+%	@deprecated Use ISO is_list/1.
 
 proper_list(List) :-
 	is_list(List).
@@ -170,21 +178,39 @@ proper_list(List) :-
 %	Return  a  list  of  unbound  variables    in   Term.  The  name
 %	term_variables/2 is more widely used.
 %	
-%	@depreciated Use term_variables/2.
+%	@deprecated Use term_variables/2.
 
 free_variables(Term, Variables) :-
 	term_variables(Term, Variables).
 
 %%	checklist(:Goal, +List)
 %	
-%	@depreciated Use maplist/2
+%	@deprecated Use maplist/2
 
 :- module_transparent
-	checklist/2.
+	checklist/2,
+	sublist/3.
 
 checklist(Goal, List) :-
 	maplist(Goal, List).
 
+%%	sublist(:Goal, +List1, ?List2)
+%	
+%	Succeeds if List2 unifies with a list holding those terms for wich
+%	call(Goal, Elem) succeeds.
+%	
+%	@deprecated Use include/3 from library(apply)
+%	@compat	DEC10 library
+
+sublist(_, [], []) :- !.
+sublist(Goal, [H|T], Sub) :-
+	call(Goal, H), !, 
+	Sub = [H|R], 
+	sublist(Goal, T, R).
+sublist(Goal, [_|T], R) :-
+	sublist(Goal, T, R).
+
+
 %%	strip_module(+Term, -Module, -Plain)
 %	
 %	This used to be an internal predicate.  It was added to the XPCE
@@ -192,7 +218,7 @@ checklist(Goal, List) :-
 %	places. From 5.4.1 onwards strip_module/3 is  built-in and the $
 %	variation is added here for compatibility.
 %	
-%	@depreciated Use strip_module/3.
+%	@deprecated Use strip_module/3.
 
 :- module_transparent
 	'$strip_module'/3.
@@ -207,7 +233,7 @@ checklist(Goal, List) :-
 %	returned  as a  SWI-Prolog string object  (see section  4.23).   See
 %	also convert_time/8.
 %	
-%	@depreciated Use format_time/3.
+%	@deprecated Use format_time/3.
 
 
 convert_time(Stamp, String) :-
@@ -224,7 +250,7 @@ convert_time(Stamp, String) :-
 %	or  might always be 0, depending  on the timing capabilities of  the
 %	system.  See also convert_time/2.
 %	
-%	@depreciated Use stamp_date_time/3.
+%	@deprecated Use stamp_date_time/3.
 
 convert_time(Stamp, Y, Mon, Day, Hour, Min, Sec, MilliSec) :-
 	stamp_date_time(Stamp,
@@ -232,5 +258,70 @@ convert_time(Stamp, Y, Mon, Day, Hour, Min, Sec, MilliSec) :-
 			     Hour, Min, FSec,
 			     _, _, _),
 			local),
-	Sec is float_integer_part(FSec),
+	Sec is integer(float_integer_part(FSec)),
 	MilliSec is integer(float_fractional_part(FSec)*1000).
+
+%%	'C'(?List, ?Head, ?Tail) is det.
+%
+%	Used to be generated by DCG.  Some people appear to be using in
+%	in normal code too.
+%	
+%	@deprecated Do not use in normal code; DCG no longer generates it.
+
+'C'([H|T], H, T).
+
+
+%%	current_thread(?Thread, ?Status) is nondet.
+%
+%	@deprecated Replaced by thread_property/2
+
+current_thread(Thread, Status) :-
+	nonvar(Thread), !,
+	catch(thread_property(Thread, status(Status)),
+	      error(existence_error(thread, _), _),
+	      fail).
+current_thread(Thread, Status) :-
+	thread_property(Thread, status(Status)).
+
+%%	current_mutex(?Mutex, ?Owner, ?Count) is nondet.
+%
+%	@deprecated Replaced by mutex_property/2
+
+current_mutex(Mutex, Owner, Count) :-
+	nonvar(Mutex), !,
+	catch(mutex_property(Mutex, status(Status)),
+	      error(existence_error(mutex, _), _),
+	      fail),
+	map_mutex_status(Status, Owner, Count).
+current_mutex(Mutex, Owner, Count) :-
+	mutex_property(Mutex, status(Status)),
+	map_mutex_status(Status, Owner, Count).
+
+map_mutex_status(unlocked, [], 0).
+map_mutex_status(locked(Owner, Count), Owner, Count).
+
+
+%%	message_queue_size(+Queue, -Size) is det.
+%
+%	True if Queue holds Size terms.
+%	
+%	@deprecated Please use message_queue_property(Queue, Size)
+
+message_queue_size(Queue, Size) :-
+	message_queue_property(Queue, size(Size)).
+
+%%	lock_predicate(+Name, +Arity) is det.
+%%	unlock_predicate(+Name, +Arity) is det.
+%
+%	@deprecated see lock_predicate/1 and unlock_predicate/1.
+
+:- module_transparent
+	lock_predicate/2,
+	unlock_predicate/2.
+
+lock_predicate(Name, Arity) :-
+	lock_predicate(Name/Arity).
+
+unlock_predicate(Name, Arity) :-
+	unlock_predicate(Name/Arity).
+
