@@ -1,4 +1,4 @@
-/*  $Id: prolog_clause.pl,v 1.15 2007/02/28 13:17:49 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
@@ -105,7 +105,10 @@ unify_term(_:X, Y) :-
 unify_term(X, _:Y) :-
 	unify_term(X, Y), !.
 unify_term(X, Y) :-
-	format('[INTERNAL ERROR: Diff: ~q <-> ~q]~n', [X, Y]),
+	format('[INTERNAL ERROR: Diff:~n'),
+	portray_clause(X),
+	format('~N*** <->~n'),
+	portray_clause(Y),
 	break.
 
 unify_args(N, N, _, _) :- !.
@@ -588,8 +591,8 @@ predicate_name(Predicate, PName) :-
 	->  true
 	;   functor(Head, Name, Arity),
 	    (   hidden_module(Module)
-	    ->  sformat(PName, '~q/~d', [Name, Arity])
-	    ;   sformat(PName, '~q:~q/~d', [Module, Name, Arity])
+	    ->  format(string(PName), '~q/~d', [Name, Arity])
+	    ;   format(string(PName), '~q:~q/~d', [Module, Name, Arity])
 	    )
 	).
 
@@ -603,7 +606,7 @@ clause_name(Ref, Name) :-
 	nth_clause(Head, N, Ref), !,
 	predicate_name(Head, PredName),
 	thaffix(N, Th),
-	sformat(Name, '~d-~w clause of ~w', [N, Th, PredName]).
+	format(string(Name), '~d-~w clause of ~w', [N, Th, PredName]).
 clause_name(_, '<meta-call>').
 
 
