@@ -1,4 +1,4 @@
-/*  $Id: rdf_db.h,v 1.53 2007/07/09 13:56:38 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
@@ -147,6 +147,7 @@ typedef struct literal
     } term;				/* external record */
   } value;
   atom_t	type_or_lang;		/* Type or language for literals */
+  unsigned int  hash;			/* saved hash */
   unsigned	objtype : 3;
   unsigned	qualifier : 2;		/* Lang/Type qualifier */
   unsigned	shared : 1;		/* member of shared table */
@@ -160,7 +161,10 @@ typedef struct literal
 
 typedef struct triple
 { atom_t	subject;
-  predicate*	predicate;
+  union
+  { predicate*	r;			/* resolved: normal DB */
+    atom_t	u;			/* used by rdf_load_db_/3 */
+  } predicate;
   union
   { literal *	literal;
     atom_t	resource;
@@ -171,6 +175,7 @@ typedef struct triple
   struct triple*next[INDEX_TABLES];	/* hash-table next links */
 					/* flags */
   unsigned	object_is_literal : 1;	/* Object is a literal */
+  unsigned	resolve_pred : 1;	/* predicates needs to be resolved */
   unsigned	indexed : 3;		/* Partials: BY_* */
   unsigned	erased  : 1;		/* If TRUE, triple is erased */
   unsigned	first   : 1;		/* I'm the first on subject */
