@@ -1,4 +1,4 @@
-/*  $Id: prolog_colour.pl,v 1.100 2007/07/09 14:51:45 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -231,8 +231,6 @@ syntax_error(E) :-
 %	style checking options. Src is the  canonical source as required
 %	by the cross-referencer.
 
-fix_operators('$:-'(_), _) :- !,	% deal with swi('boot/init.pl')
-	style_check(+dollar).
 fix_operators((:- Directive), Src) :- !,
 	process_directive(Directive, Src).
 fix_operators(_, _).
@@ -241,10 +239,6 @@ process_directive(style_check(X), _) :- !,
 	style_check(X).
 process_directive(system_mode(on), _) :- !,
 	style_check(+dollar).
-process_directive($(Name), _) :-
-	atom(Name),
-	style_check(+dollar),
-	fail.				% allow for other expansions
 process_directive(op(P,T,N), _) :- !,
 	catch(emacs_push_op(P, T, N), _, true).
 process_directive(module(_Name, Export), _) :- !,
@@ -281,7 +275,7 @@ process_use_module(File, Src) :-
 
 colourise(TB, Fd) :-
 	character_count(Fd, Pos),
-	emacs_prolog_mode:read_term_from_stream(Fd, Pos,
+	emacs_prolog_mode:read_term_from_stream(TB, Fd, Pos,
 						Term,
 						Error,
 						_Singletons,
@@ -487,6 +481,10 @@ colourise_dcg_subgoals([Pos|T], N, Body, Origin, TB) :-
 	NN is N + 1,
 	colourise_dcg_subgoals(T, NN, Body, Origin, TB).
 
+dcg_extend(Term, _) :-
+	var(Term), !, fail.
+dcg_extend(M:Term, M:Goal) :-
+	dcg_extend(Term, Goal).
 dcg_extend(Term, Goal) :-
 	callable(Term),
 	Term =.. List,
@@ -597,6 +595,9 @@ instantiate_meta([H|T]) :-
 
 expand_meta(MetaSpec, Goal, Goal) :-
 	MetaSpec == 0.
+expand_meta(MetaSpec, M:Goal, M:Expanded) :-
+	atom(M), !,
+	expand_meta(MetaSpec, Goal, Expanded).
 expand_meta(MetaSpec, Goal, Expanded) :-
 	integer(MetaSpec),
 	callable(Goal), !,
@@ -739,11 +740,11 @@ colourise_exports2(_, _, _).
 %	Colourise import list from use_module/2, importing from File.
 
 colourise_imports(List, File, TB, Pos) :-
-	(   xref_public_list(File, _, Public, TB)
+	(   xref_public_list(File, Path, Public, TB)
 	->  true
 	;   Public = []
 	),
-	colourise_imports(List, File, Public, TB, Pos).
+	colourise_imports(List, Path, Public, TB, Pos).
 
 colourise_imports([], _, _, _, _).
 colourise_imports(List, File, Public, TB, list_position(_,_,ElmPos,Tail)) :- !,
@@ -752,17 +753,34 @@ colourise_imports(List, File, Public, TB, list_position(_,_,ElmPos,Tail)) :- !,
 	;   colour_item(type_error(list), TB, Tail)
 	),
 	colourise_imports2(List, File, Public, TB, ElmPos).
+colourise_imports(except(Except), File, Public, TB,
+		  term_position(_,_,FF,FT,[LP])) :- !,
+	colour_item(keyword(except), TB, FF-FT),
+	colourise_imports(Except, File, Public, TB, LP).
 colourise_imports(_, _, _, TB, Pos) :-
 	colour_item(type_error(list), TB, Pos).	
 
 colourise_imports2([G0|GT], File, Public, TB, [P0|PT]) :- !,
-	colourise_declaration(G0, TB, P0),
+	colourise_import(G0, File, TB, P0),
 	colourise_imports2(GT, File, Public, TB, PT).
 colourise_imports2(_, _, _, _, _).
 
+
+colourise_import(PI as Name, File, TB, term_position(_,_,FF,FT,[PP,NP])) :-
+	pi_to_term(PI, Goal), !,
+	colour_item(goal(imported(File), Goal), TB, PP),
+	functor(Goal, _, Arity),
+	functor(NewGoal, Name, Arity),
+	goal_classification(TB, NewGoal, [], Class),
+	colour_item(goal(Class, NewGoal), TB, NP),
+	colour_item(keyword(as), TB, FF-FT).
+colourise_import(PI, _, TB, Pos) :-
+	colourise_declaration(PI, TB, Pos).
+
+
 %	colourise_declarations(+Term, +TB, +Pos)
 %	
-%	Colourise the Name/Arity lists of dynamic, multifile, etc
+%	Colourise the Predicate indicator lists of dynamic, multifile, etc
 %	declarations.
 
 colourise_declarations((Head,Tail), TB,
@@ -772,27 +790,27 @@ colourise_declarations((Head,Tail), TB,
 colourise_declarations(Last, TB, Pos) :-
 	colourise_declaration(Last, TB, Pos).
 
-colourise_declaration($(Name)/Arity, TB, Pos) :-
-	atom(Name),
-	style_check(?(dollar)), !,	% deal with system boot-files
-	atom_concat($, Name, TheName),
-	colourise_declaration(TheName/Arity, TB, Pos).
-colourise_declaration(Name/Arity, TB, Pos) :-
-	atom(Name), integer(Arity), !,
-	functor(Goal, Name, Arity),
+colourise_declaration(PI, TB, Pos) :-
+	pi_to_term(PI, Goal), !,
 	goal_classification(TB, Goal, [], Class),
 	colour_item(goal(Class, Goal), TB, Pos).
-colourise_declaration(Module:Name/Arity, TB,
+colourise_declaration(Module:PI, TB,
 		      term_position(_,_,_,_,[PM,PG])) :-
-	atom(Module), atom(Name), integer(Arity), !,
+	atom(Module), pi_to_term(PI, Goal), !,
 	colour_item(module(M), TB, PM),
-	functor(Goal, Name, Arity),
 	colour_item(goal(extern(M), Goal), TB, PG).
 colourise_declaration(op(_,_,_), TB, Pos) :-
 	colour_item(exported_operator, TB, Pos).
 colourise_declaration(_, TB, Pos) :-
 	colour_item(type_error(export_declaration), TB, Pos).
 
+pi_to_term(Name/Arity, Term) :-
+	atom(Name), integer(Arity), !,
+	functor(Term, Name, Arity).
+pi_to_term(Name//Arity0, Term) :-
+	atom(Name), integer(Arity0), !,
+	Arity is Arity0 + 2,
+	functor(Term, Name, Arity).
 
 %	colour_item(+Class, +TB, +Pos)
 %
@@ -883,7 +901,7 @@ goal_classification(Goal, built_in) :-
 	built_in_predicate(Goal), !.
 goal_classification(Goal, autoload) :-	% SWI-Prolog
 	functor(Goal, Name, Arity),
-	'$in_library'(Name, Arity), !.
+	'$in_library'(Name, Arity, _Path), !.
 goal_classification(Goal, global) :-	% SWI-Prolog
 	current_predicate(_, user:Goal), !.
 goal_classification(SS, expanded) :-	% XPCE (TBD)
@@ -911,6 +929,10 @@ classify_head(_TB, _Goal, undefined).
 built_in_predicate(Goal) :-
 	predicate_property(system:Goal, built_in), !.
 built_in_predicate(module(_, _)).
+built_in_predicate(if(_)).
+built_in_predicate(elif(_)).
+built_in_predicate(else).
+built_in_predicate(endif).
 
 %	Specify colours for individual goals.  Currently used only to
 %	highlight file references, so we can jump to them and are indicated
@@ -919,6 +941,8 @@ built_in_predicate(module(_, _)).
 goal_colours(module(_,_),	     built_in-[identifier,exports]).
 goal_colours(use_module(_),	     built_in-[file]).
 goal_colours(use_module(File,_),     built_in-[file,imports(File)]).
+goal_colours(reexport(_),	     built_in-[file]).
+goal_colours(reexport(File,_),       built_in-[file,imports(File)]).
 goal_colours(dynamic(_),	     built_in-[predicates]).
 goal_colours(thread_local(_),	     built_in-[predicates]).
 goal_colours(multifile(_),	     built_in-[predicates]).
@@ -1017,7 +1041,7 @@ def_style(head(hook),	  	style(underline  := @on, colour := blue)).
 def_style(head(meta),	  	@default).
 def_style(head(constraint(_)),	style(bold := @on, colour := darkcyan)).
 def_style(head(_),	  	style(bold := @on)).
-
+def_style(module(_),		style(colour := dark_slate_blue)).
 def_style(comment,		style(colour := dark_green)).
 
 def_style(directive,	  	style(background := grey90)).
@@ -1043,6 +1067,7 @@ def_style(class(undefined,_),	style(colour := red,
 def_style(prolog_data,		style(colour := blue,
 				      underline  := @on)).
 
+def_style(keyword(_), 		style(colour := blue)).
 def_style(identifier, 		style(bold := @on)).
 def_style(delimiter,		style(bold := @on)).
 def_style(expanded,		style(colour := blue,
@@ -1505,10 +1530,11 @@ identify(F) :->
 identify_pred(Term, _, Summary) :-
 	identify(Term, String),
 	new(Summary, string(String)).
-identify_pred(built_in, F, Summary) :-	% SWI-Prolog documented built-in
-	get(F, predicate, Pred),
+identify_pred(Class, F, Summary) :-		% SWI-Prolog documented built-in
+	get(F, predicate, Pred),	% & PlDoc summaries
 	get(Pred, summary, Summary0), !,
-	new(Summary, string('%N: %s', Pred, Summary0)).
+	functor(Class, ClassName, _),
+	new(Summary, string('%N: [%s] %s', Pred, ClassName, Summary0)).
 identify_pred(built_in, F, Summary) :-
 	get(F, head, Head),
 	predicate_property(system:Head, foreign), !,
@@ -1781,6 +1807,8 @@ identify_fragment(head(exported), _, 'Exported (Public) predicate').
 identify_fragment(head(multifile), _, 'Multifile predicate').
 identify_fragment(head(constraint), _, 'Constraint').
 identify_fragment(prolog_data, _, 'Pass Prolog term unmodified').
+identify_fragment(keyword(except), _, 'Import all except given').
+identify_fragment(keyword(as), _, 'Import under a different name').
 identify_fragment(Class, _, Summary) :-
 	term_to_atom(Class, Summary).
 
