@@ -1,11 +1,11 @@
-/*  $Id: pl-file.c,v 1.216 2007/07/02 20:33:53 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        jan@swi.psy.uva.nl
+    E-mail:        wielemak@science.uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2002, University of Amsterdam
+    Copyright (C): 1985-2007, University of Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -578,6 +578,8 @@ getOutputStream(term_t t, IOSTREAM **stream)
   } else if ( PL_get_atom(t, &a) && a == ATOM_user )
   { *stream = getStream(Suser_output);
     return TRUE;
+  } else
+  { *stream = NULL;			/* make compiler happy */
   }
 
   if ( !PL_get_stream_handle(t, &s) )
@@ -605,6 +607,8 @@ getInputStream__LD(term_t t, IOSTREAM **stream ARG_LD)
   } else if ( PL_get_atom(t, &a) && a == ATOM_user )
   { *stream = getStream(Suser_input);
     return TRUE;
+  } else
+  { *stream = NULL;			/* make compiler happy */
   }
 
   if ( !get_stream_handle(t, &s, SH_ERRORS|SH_ALIAS) )
@@ -1071,8 +1075,31 @@ function will read the first character and  then skip all character upto
 and including the newline.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
+static int
+Sgetcode_intr(IOSTREAM *s, int signals)
+{ int c;
+
+#ifdef __WINDOWS__
+  int newline = s->newline;
+  s->newline = SIO_NL_POSIX;		/* avoid blocking \r */
+#endif
+  do
+  { Sclearerr(s);
+    c = Sgetcode(s);
+  } while ( c == -1 && 
+	    errno == EINTR &&
+	    (!signals || PL_handle_signals() >= 0)
+	  );
+#ifdef __WINDOWS__
+  s->newline = newline;
+#endif
+
+  return c;
+}
+
+
 int
-getSingleChar(IOSTREAM *stream)
+getSingleChar(IOSTREAM *stream, int signals)
 { GET_LD
   int c;
   ttybuf buf;
@@ -1085,19 +1112,19 @@ getSingleChar(IOSTREAM *stream)
   if ( !trueFeature(TTY_CONTROL_FEATURE) )
   { int c2;
 
-    c2 = Sgetcode(stream);
+    c2 = Sgetcode_intr(stream, signals);
     while( c2 == ' ' || c2 == '\t' )	/* skip blanks */
-      c2 = Sgetcode(stream);
+      c2 = Sgetcode_intr(stream, signals);
     c = c2;
     while( c2 != EOF && c2 != '\n' )	/* read upto newline */
-      c2 = Sgetcode(stream);
+      c2 = Sgetcode_intr(stream, signals);
   } else
   { if ( stream->position )
     { IOPOS oldpos = *stream->position;
-      c = Sgetcode(stream);
+      c = Sgetcode_intr(stream, signals);
       *stream->position = oldpos;
     } else
-      c = Sgetcode(stream);
+      c = Sgetcode_intr(stream, signals);
   }
 
   if ( c == 4 || c == 26 )		/* should ask the terminal! */
@@ -1401,6 +1428,27 @@ pl_set_stream(term_t stream, term_t attr)
 	  goto error;
 	}
 	goto ok;
+      } else if ( aname == ATOM_newline )
+      { atom_t val;
+
+	if ( !PL_get_atom_ex(a, &val) )
+	  goto error;
+	if ( val == ATOM_posix )
+	  s->newline = SIO_NL_POSIX;
+	else if ( val == ATOM_dos )
+	  s->newline = SIO_NL_DOS;
+	else if ( val == ATOM_detect )
+	{ if ( false(s, SIO_INPUT) )
+	  { PL_error(NULL, 0, "detect only allowed for input streams",
+		     ERR_DOMAIN, ATOM_newline, a);
+	    goto error;
+	  }
+	  s->newline = SIO_NL_DETECT;
+	} else
+	{ PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_newline, a);
+	  goto error;
+	}
+	goto ok;
       }
     }
   }
@@ -1646,16 +1694,19 @@ PRED_IMPL("read_pending_input", 3, read_pending_input, 0)
     n = Sread_pending(s, buf, sizeof(buf), 0);
     if ( n < 0 )			/* should not happen */
       return streamStatus(s);
+    if ( n == 0 )
+      return PL_unify(A2, A3);
 
-    gstore = allocGlobal(n*3);
+    gstore = allocGlobal(n*3);		/* TBD: shift */
     lp = valTermRef(A2);
     deRef(lp);
     tp = valTermRef(A3);
     deRef(tp);
 
     if ( !isVar(*lp) )
-      return PL_error(NULL, 0, NULL, ERR_INSTANTIATION, A2);
+      return PL_error(NULL, 0, NULL, ERR_MUST_BE_VAR, 2, A2);
     *lp = consPtr(gstore, TAG_COMPOUND|STG_GLOBAL);
+    Trail(lp);
 
     for(i=0; i<n; )
     { *gstore++ = FUNCTOR_dot2;
@@ -1896,7 +1947,7 @@ word
 pl_get_single_char(term_t chr)
 { GET_LD
   IOSTREAM *s = getStream(Suser_input);
-  int c = getSingleChar(s);
+  int c = getSingleChar(s, TRUE);
 
   if ( c == EOF )
   { PL_unify_integer(chr, -1);
@@ -2095,7 +2146,7 @@ pl_prompt1(term_t prompt)
 
   if ( PL_get_atom(prompt, &a) )
   { prompt1(a);
-  } else if ( PL_get_text_ex(prompt, &txt,  CVT_ALL) )
+  } else if ( PL_get_text(prompt, &txt,  CVT_ALL|CVT_EXCEPTION) )
   { prompt1(textToAtom(&txt));
   } else
     fail;
@@ -2167,7 +2218,9 @@ pl_tab2(term_t out, term_t spaces)
   if ( rval )
     return streamStatus(s);
 
+#ifdef O_GMP
 error:
+#endif
   (void)streamStatus(s);
   fail;
 }
@@ -2359,7 +2412,7 @@ openStream(term_t file, term_t mode, term_t options)
 
 					/* FILE */
   if ( PL_get_chars(file, &path,
-		    CVT_ATOM|CVT_STRING|CVT_INTEGER|CVT_EXCEPTION|REP_FN) )
+		    CVT_ATOM|CVT_STRING|CVT_EXCEPTION|REP_FN) )
   { if ( !(s = Sopen_file(path, how)) )
     { PL_error(NULL, 0, OsError(), ERR_FILE_OPERATION,
 	       ATOM_open, ATOM_source_sink, file);
@@ -2658,7 +2711,12 @@ pl_open_null_stream(term_t stream)
 { int sflags = SIO_NBUF|SIO_RECORDPOS|SIO_OUTPUT;
   IOSTREAM *s = Snew((void *)NULL, sflags, (IOFUNCTIONS *)&nullFunctions);
 
-  return PL_unify_stream_or_alias(stream, s);
+  if ( s )
+  { s->encoding = ENC_UTF8;
+    return PL_unify_stream_or_alias(stream, s);
+  }
+
+  fail;
 }
 
 
@@ -2914,6 +2972,20 @@ stream_bom_prop(IOSTREAM *s, term_t prop ARG_LD)
 }
 
 
+static int
+stream_newline_prop(IOSTREAM *s, term_t prop ARG_LD)
+{ switch ( s->newline ) 
+  { case SIO_NL_POSIX:
+    case SIO_NL_DETECT:
+      return PL_unify_atom(prop, ATOM_posix);
+    case SIO_NL_DOS:
+      return PL_unify_atom(prop, ATOM_dos);
+  }
+
+  fail;
+}
+
+
 static int
 stream_encoding_prop(IOSTREAM *s, term_t prop ARG_LD)
 { return PL_unify_atom(prop, encoding_to_atom(s->encoding));
@@ -2973,6 +3045,7 @@ static const sprop sprop_list [] =
   { FUNCTOR_tty1,	    stream_tty_prop },
   { FUNCTOR_encoding1,	    stream_encoding_prop },
   { FUNCTOR_bom1,	    stream_bom_prop },
+  { FUNCTOR_newline1,	    stream_newline_prop },
   { FUNCTOR_representation_errors1, stream_reperror_prop },
   { 0,			    NULL }
 };
@@ -3682,6 +3755,8 @@ add_option(term_t options, functor_t f, atom_t val)
   PL_reset_term_refs(head);
 }
 
+#define CVT_FILENAME (CVT_ATOM|CVT_STRING|CVT_LIST)
+
 int
 PL_get_file_name(term_t n, char **namep, int flags)
 { GET_LD
@@ -3716,10 +3791,10 @@ PL_get_file_name(term_t n, char **namep, int flags)
   }
 
   if ( flags & PL_FILE_NOERRORS )
-  { if ( !PL_get_chars(n, &name, CVT_ALL|REP_FN) )
+  { if ( !PL_get_chars(n, &name, CVT_FILENAME|REP_FN) )
       fail;
   } else
-  { if ( !PL_get_chars_ex(n, &name, CVT_ALL|REP_FN) )
+  { if ( !PL_get_chars_ex(n, &name, CVT_FILENAME|REP_FN) )
       fail;
   }
 
@@ -3768,7 +3843,7 @@ pl_time_file(term_t name, term_t t)
   { intptr_t time;
 
     if ( (time = LastModifiedFile(fn)) == -1 )
-      fail;
+      return PL_error(NULL, 0, NULL, ERR_FILE_OPERATION, ATOM_time, ATOM_file, name);
 
     return unifyTime(t, time);
   }
@@ -4224,6 +4299,7 @@ pl_copy_stream_data3(term_t in, term_t out, term_t len)
 { GET_LD
   IOSTREAM *i, *o;
   int c;
+  int count = 0;
 
   if ( !getInputStream(in, &i) )
     return FALSE;
@@ -4234,7 +4310,12 @@ pl_copy_stream_data3(term_t in, term_t out, term_t len)
 
   if ( !len )
   { while ( (c = Sgetcode(i)) != EOF )
-    { if ( Sputcode(c, o) < 0 )
+    { if ( (++count % 4096) == 0 && PL_handle_signals() < 0 )
+      { releaseStream(i);
+	releaseStream(o);
+	fail;
+      }
+      if ( Sputcode(c, o) < 0 )
       { releaseStream(i);
 	return streamStatus(o);
       }
@@ -4246,7 +4327,12 @@ pl_copy_stream_data3(term_t in, term_t out, term_t len)
       fail;
     
     while ( n-- > 0 && (c = Sgetcode(i)) != EOF )
-    { if ( Sputcode(c, o) < 0 )
+    { if ( (++count % 4096) == 0 && PL_handle_signals() < 0 )
+      { releaseStream(i);
+	releaseStream(o);
+	fail;
+      }
+      if ( Sputcode(c, o) < 0 )
       { releaseStream(i);
 	return streamStatus(o);
       }
@@ -4270,24 +4356,24 @@ pl_copy_stream_data2(term_t in, term_t out)
 BeginPredDefs(file)
   PRED_DEF("set_prolog_IO", 3, set_prolog_IO, 0)
   PRED_DEF("read_pending_input", 3, read_pending_input, 0)
-  PRED_DEF("get_code", 2, get_code2, 0)
-  PRED_DEF("get_code", 1, get_code1, 0)
-  PRED_DEF("get_char", 2, get_char2, 0)
-  PRED_DEF("get_char", 1, get_char1, 0)
-  PRED_DEF("get_byte", 2, get_byte2, 0)
-  PRED_DEF("get_byte", 1, get_byte1, 0)
-  PRED_DEF("peek_code", 2, peek_code2, 0)
-  PRED_DEF("peek_code", 1, peek_code1, 0)
-  PRED_DEF("peek_char", 2, peek_char2, 0)
-  PRED_DEF("peek_char", 1, peek_char1, 0)
-  PRED_DEF("peek_byte", 2, peek_byte2, 0)
-  PRED_DEF("peek_byte", 1, peek_byte1, 0)
-  PRED_DEF("put_byte", 2, put_byte2, 0)
-  PRED_DEF("put_byte", 1, put_byte1, 0)
-  PRED_DEF("put_code", 2, put_code2, 0)
-  PRED_DEF("put_code", 1, put_code1, 0)
-  PRED_DEF("put_char", 2, put_code2, 0)
-  PRED_DEF("put_char", 1, put_code1, 0)
+  PRED_DEF("get_code", 2, get_code2, PL_FA_ISO)
+  PRED_DEF("get_code", 1, get_code1, PL_FA_ISO)
+  PRED_DEF("get_char", 2, get_char2, PL_FA_ISO)
+  PRED_DEF("get_char", 1, get_char1, PL_FA_ISO)
+  PRED_DEF("get_byte", 2, get_byte2, PL_FA_ISO)
+  PRED_DEF("get_byte", 1, get_byte1, PL_FA_ISO)
+  PRED_DEF("peek_code", 2, peek_code2, PL_FA_ISO)
+  PRED_DEF("peek_code", 1, peek_code1, PL_FA_ISO)
+  PRED_DEF("peek_char", 2, peek_char2, PL_FA_ISO)
+  PRED_DEF("peek_char", 1, peek_char1, PL_FA_ISO)
+  PRED_DEF("peek_byte", 2, peek_byte2, PL_FA_ISO)
+  PRED_DEF("peek_byte", 1, peek_byte1, PL_FA_ISO)
+  PRED_DEF("put_byte", 2, put_byte2, PL_FA_ISO)
+  PRED_DEF("put_byte", 1, put_byte1, PL_FA_ISO)
+  PRED_DEF("put_code", 2, put_code2, PL_FA_ISO)
+  PRED_DEF("put_code", 1, put_code1, PL_FA_ISO)
+  PRED_DEF("put_char", 2, put_code2, PL_FA_ISO)
+  PRED_DEF("put_char", 1, put_code1, PL_FA_ISO)
   PRED_DEF("put", 2, put2, 0)
   PRED_DEF("put", 1, put1, 0)
   PRED_DEF("skip", 1, skip1, 0)
