@@ -1,4 +1,4 @@
-/*  $Id: pl-global.h,v 1.91 2007/06/28 15:05:06 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
@@ -134,7 +134,9 @@ typedef struct
   } arith;
 
   struct
-  { buffer	array;			/* index --> atom */
+  { Atom *	array;			/* index --> atom */
+    size_t	count;			/* elements in array */
+    size_t	array_allocated;	/* allocated size of array */
     unsigned int buckets;		/* # buckets in char * --> atom */
     Atom *	table;			/* hash-table */
     int		lookups;		/* # atom lookups */
@@ -146,7 +148,7 @@ typedef struct
     size_t	margin;			/* # atoms to grow before collect */
     size_t	non_garbage;		/* # atoms for after last AGC */
     int		gc;			/* # atom garbage collections */
-    long	collected;		/* # collected atoms */
+    int64_t	collected;		/* # collected atoms */
     real	gc_time;		/* Time spent on atom-gc */
     PL_agc_hook_t gc_hook;		/* Current hook */
 #endif
@@ -175,6 +177,13 @@ typedef struct
     int		  _loaded;		/* system extensions are loaded */
   } foreign;
 
+#ifdef O_GMP
+  struct
+  { int initialised;			/* is GMP initialised? */
+    int keep_alloc_functions;		/* do not change allocation */
+  } gmp;
+#endif
+
   struct				/* pl-format.c */
   { Table	predicates;
   } format;
@@ -225,13 +234,16 @@ typedef struct
     Procedure	prolog_trace_interception4;
     Procedure	portray;		/* portray/1 */
     Procedure   dcall1;			/* $call/1 */
-    Procedure	call_cleanup3;		/* call_cleanup/2 */
+    Procedure	setup_and_call_cleanup4; /* setup_and_call_cleanup/4 */
     Procedure	undefinterc4;		/* $undefined_procedure/4 */
     Procedure   dthread_init0;		/* $thread_init/0 */
 #ifdef O_ATTVAR
     Procedure	dwakeup1;		/* system:$wakeup/1 */
     Procedure	portray_attvar1;	/* $attvar:portray_attvar/1 */
 #endif
+#ifdef O_CALL_RESIDUE
+    Procedure	call_residue_vars2;	/* $attvar:call_residue_vars/2 */
+#endif
 
     SourceFile  reloading;		/* source file we are re-loading */
     int		active_marked;		/* #prodedures marked active */
@@ -299,6 +311,7 @@ typedef struct PL_local_data
   uintptr_t	pending_signals;	/* PL_raise() pending signals */
   record_t	pending_exception;	/* Pending exception from signal */
   int		current_signal;		/* Currently handled signal */
+  int		sync_signal;		/* Current signal is synchronous */
   int		critical;		/* heap is being modified */
   int		aborted;		/* thread asked for abort */
   Stack		outofstack;		/* thread is out of stack */
@@ -360,6 +373,15 @@ typedef struct PL_local_data
     double	system_cputime;		/* Kernel saved CPU time */
   } statistics;
 
+#ifdef O_GMP
+  struct
+  { size_t	allocated;		/* memory allocated */
+    ar_context *context;		/* current allocation context */
+    mp_mem_header *head;		/* linked list of allocated chunks */
+    mp_mem_header *tail;
+  } gmp;
+#endif
+
 #ifdef O_PROFILE
   struct
   { int		active;			/* profiler is on */
@@ -384,10 +406,6 @@ typedef struct PL_local_data
   { intptr_t 	generator;		/* See PL_atom_generator() */
   } atoms;
 
-  struct
-  { struct assoc* bags;			/* findall/setof bags storage */
-  } bags;
-
   struct
   { VarDef *	vardefs;		/* compiler variable analysis */
     int		nvardefs;
@@ -421,12 +439,19 @@ typedef struct PL_local_data
   { Table	  table;		/* Feature table */
     pl_features_t mask;			/* Masked access to booleans */
     int		  write_attributes;	/* how to write attvars? */
+    occurs_check_t occurs_check;	/* Unify and occurs check */
   } feature;
 
   struct
   { FindData	find;			/* /<ports> <goal> in tracer */
   } trace;
 
+#ifdef O_ATOMGC
+  struct
+  { struct findall_bag *bags;		/* Known bags */
+  } bags;
+#endif
+
   struct
   { AbortHandle	_abort_head;		/* PL_abort_hook() */
     AbortHandle _abort_tail;
@@ -470,8 +495,12 @@ typedef struct PL_local_data
     Word *_mark_top;			/* Top of this array */
     Table _check_table;			/* relocation address table */
 #endif
+    int64_t inferences;			/* #inferences at last GC */
 
     pl_gc_status_t	status;		/* Garbage collection status */
+#ifdef O_CALL_RESIDUE
+    int			marked_attvars;	/* do not GC attvars */
+#endif
   } gc;
 
 #ifdef O_SHIFT_STACKS
@@ -521,7 +550,6 @@ GLOBAL PL_local_data_t *PL_current_engine_ptr;
 #define hBase			(GD->base_of_heap)
 #define heap_base		(GD->rounded_heap_base)
 #define functor_array		(GD->functors.array)
-#define atom_array		(GD->atoms.array)
 #define systemDefaults		(GD->defaults)
 #define features		(LD->feature.mask)
 
