@@ -1,4 +1,4 @@
-/*  $Id: pl-incl.h,v 1.279 2007/06/19 09:05:31 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
@@ -148,6 +148,7 @@ handy for it someone wants to add a data type to the system.
 #define O_ATOMGC		1
 #define O_CLAUSEGC		1
 #define O_ATTVAR		1
+#define O_CALL_RESIDUE		1
 #define O_GVAR			1
 #define O_CYCLIC		1
 #ifdef HAVE_GMP_H
@@ -461,6 +462,8 @@ them.  Descriptions:
 				/* Prolog's integer range */
 #define PLMINTAGGEDINT		(-(intptr_t)((word)1<<(WORDBITSIZE-LMASK_BITS-1)))
 #define PLMAXTAGGEDINT		(-PLMINTAGGEDINT - 1)
+#define PLMINTAGGEDINT32	(-(intptr_t)((word)1<<(32-LMASK_BITS-1)))
+#define PLMAXTAGGEDINT32	(-PLMINTAGGEDINT32 - 1)
 #define inTaggedNumRange(n)	(((n)&~PLMAXTAGGEDINT) == 0 || \
 				 ((n)&~PLMAXTAGGEDINT) == ~PLMAXTAGGEDINT)
 #define PLMININT		(((int64_t)-1<<(INT64BITSIZE-1)))
@@ -946,6 +949,11 @@ typedef struct
 #define GP_HIDESYSTEM	0x200		/* hide system module */
 #define GP_TYPE_QUIET	0x400		/* don't throw errors on wrong types */
 #define GP_EXISTENCE_ERROR 0x800	/* throw error if proc is not found */
+#define GP_QUALIFY	0x1000		/* Always module-qualify */
+
+					/* get_functor() */
+#define GF_EXISTING	1
+#define GF_PROCEDURE	2		/* check for max arity */
 
 
 		 /*******************************
@@ -1011,6 +1019,7 @@ with one operation, it turns out to be faster as well.
 #define P_THREAD_LOCAL		(0x01000000L) /* predicate */
 #define P_FOREIGN_CREF		(0x02000000L) /* predicate */
 #define P_DIRTYREG		(0x04000000L) /* predicate */
+#define P_ISO			(0x08000000L) /* predicate */
 
 #define ERASED			(0x0001) /* clause, record */
 					 /* clause flags */
@@ -1035,6 +1044,7 @@ with one operation, it turns out to be faster as well.
 #define R_DIRTY			(0x0001) /* recordlist */
 #define R_EXTERNAL		(0x0002) /* record: inline atoms */
 #define R_DUPLICATE		(0x0004) /* record: include references */
+#define R_NOLOCK		(0x0008) /* record: do not lock atoms */
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Handling environment (or local stack) frames.
@@ -1193,7 +1203,7 @@ struct atom
 { Atom		next;		/* next in chain */
   word		atom;		/* as appearing on the global stack */
 #ifdef O_HASHTERM
-  int		hash_value;	/* hash-key value */
+  unsigned int  hash_value;	/* hash-key value */
 #endif
 #ifdef O_ATOMGC
   unsigned int	references;	/* reference-count */
@@ -1507,6 +1517,7 @@ struct queryFrame
   Word	       *aSave;			/* saved argument-stack */
   int		solutions;		/* # of solutions produced */
   Choice	saved_bfr;		/* Saved choice-point */
+  QueryFrame	parent;			/* Only valid inside GC */
   struct choice	choice;			/* First (dummy) choice-point */
   LocalFrame	saved_environment;	/* Parent local-frame */
 					/* Do not put anything between */
@@ -1514,6 +1525,9 @@ struct queryFrame
   struct localFrame frame;		/* The local frame */
 };
 
+#define QueryOfTopFrame(fr) \
+	(QueryFrame)((char *)(fr) - offset(queryFrame, frame))
+
 
 #define FLI_MAGIC 		82649821
 #define FLI_MAGIC_CLOSED	42424242
@@ -1650,7 +1664,7 @@ struct free_chunk
 struct alloc_pool
 { char	       *space;			/* pointer to free space */
   size_t	free;			/* size of free space */
-  intptr_t 		allocated;		/* total bytes allocated */
+  size_t 	allocated;		/* total bytes allocated */
 					/* fast perfect fit chains */
   Chunk  	free_chains[ALLOCFAST/sizeof(Chunk)+1];
   int		free_count[ALLOCFAST/sizeof(Chunk)+1];
@@ -1807,7 +1821,7 @@ typedef struct
 } sig_handler, *SigHandler;
 #endif /* HAVE_SIGNAL */
 
-#define SIG_EXCEPTION	   9		/* cannot be catched anyway */
+#define SIG_EXCEPTION	   9		/* cannot be caught anyway */
 #define SIG_ATOM_GC	  30		/* `safe' and reserved signals */
 #define SIG_THREAD_SIGNAL 31
 
@@ -1821,6 +1835,7 @@ typedef struct
 #define OPT_ATOM	(3)
 #define OPT_TERM	(4)		/* arbitrary term */
 #define OPT_LONG	(5)
+#define OPT_NATLONG	(6)		/* > 0 */
 #define OPT_TYPE_MASK	0xff
 #define OPT_INF		0x100		/* allow 'inf' */
 
@@ -2058,8 +2073,10 @@ typedef struct
 #define PROCEDURE_event_hook1		(GD->procedures.event_hook1)
 #define PROCEDURE_print_message2	(GD->procedures.print_message2)
 #define PROCEDURE_dcall1		(GD->procedures.dcall1)
-#define PROCEDURE_call_cleanup3		(GD->procedures.call_cleanup3)
+#define PROCEDURE_setup_and_call_cleanup4 \
+				(GD->procedures.setup_and_call_cleanup4)
 #define PROCEDURE_dwakeup1		(GD->procedures.dwakeup1)
+#define PROCEDURE_call_residue_vars2	(GD->procedures.call_residue_vars2)
 #define PROCEDURE_dthread_init0		(GD->procedures.dthread_init0)
 #define PROCEDURE_exception_hook4	(GD->procedures.exception_hook4)
 
@@ -2140,7 +2157,7 @@ typedef struct debuginfo
 #define FILEVARS_FEATURE	  0x02000 /* Expand $var and ~ in filename */
 #define AUTOLOAD_FEATURE	  0x04000 /* do autoloading */
 #define CHARCONVERSION_FEATURE	  0x08000 /* do character-conversion */
-#define TAILRECURSION_FEATURE	  0x10000 /* Tail recursion enabled? */
+#define LASTCALL_FEATURE	  0x10000 /* Last call optimization enabled? */
 #define EX_ABORT_FEATURE	  0x20000 /* abort with exception */
 #define BACKQUOTED_STRING_FEATURE 0x40000 /* `a string` */
 #define SIGNALS_FEATURE		  0x80000 /* Handle signals */
@@ -2154,6 +2171,13 @@ typedef struct
 #define setFeatureMask(mask)	set(&features, mask)
 #define clearFeatureMask(mask)	clear(&features, mask)
 
+typedef enum
+{ OCCURS_CHECK_FALSE = 0,
+  OCCURS_CHECK_TRUE,
+  OCCURS_CHECK_ERROR
+} occurs_check_t;
+
+
 #ifdef O_LIMIT_DEPTH
 #define DEPTH_NO_LIMIT	(~0x0L)		/* Highest value */
 #endif
@@ -2221,10 +2245,11 @@ decrease).
 #include "pl-thread.h"			/* thread manipulation */
 #include "pl-data.h"			/* Access Prolog data */
 #include "pl-segstack.h"		/* Segmented stacks */
+#include "pl-gmp.h"			/* GNU-GMP support */
 #include "pl-global.h"			/* global data */
 #include "pl-funcs.h"			/* global functions */
 #include "pl-text.h"			/* text manipulation */
-#include "pl-gmp.h"			/* GNU-GMP support */
+#include "pl-hash.h"			/* Murmurhash function */
 
 #ifdef __DECC				/* Dec C-compiler: avoid conflicts */
 #undef leave
