@@ -1,4 +1,4 @@
-/*  $Id: pl-itf.h,v 1.361 2007/07/09 15:27:09 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
@@ -25,6 +25,14 @@
 #ifndef _FLI_H_INCLUDED
 #define _FLI_H_INCLUDED
 
+#ifndef __SWI_PROLOG__			/* use this to switch on Prolog dialect */
+#define __SWI_PROLOG__			/* normally defined by the plld compiler driver */
+#endif
+
+#if defined(_MSC_VER) && !defined(__WINDOWS__)
+#define __WINDOWS__ 1
+#endif
+
 #include <stdarg.h>
 #include <stdlib.h>			/* get size_t */
 #include <stddef.h>
@@ -55,7 +63,7 @@ before loading this file.  See end of this file.
 /* PLVERSION: 10000 * <Major> + 100 * <Minor> + <Patch> */
 
 #ifndef PLVERSION
-#define PLVERSION 50637
+#define PLVERSION 50653
 #endif
 
 		 /*******************************
@@ -240,7 +248,7 @@ PL_EXPORT(void *)	PL_foreign_context_address(control_t);
 		*********************************/
 
 typedef struct PL_extension
-{ char 		*predicate_name;	/* Name of the predicate */
+{ const char   *predicate_name;		/* Name of the predicate */
   short		arity;			/* Arity of the predicate */
   pl_function_t	function;		/* Implementing functions */
   short		flags;			/* Or of PL_FA_... */
@@ -251,6 +259,7 @@ typedef struct PL_extension
 #define PL_FA_NONDETERMINISTIC	(0x04)	/* foreign is non-deterministic */
 #define PL_FA_VARARGS		(0x08)	/* call using t0, ac, ctx */
 #define PL_FA_CREF		(0x10)	/* Internal: has clause-reference */
+#define PL_FA_ISO		(0x20)	/* Internal: ISO core predicate */
 
 extern			PL_extension PL_extensions[]; /* not Win32! */
 PL_EXPORT(void)		PL_register_extensions(const PL_extension *e);
@@ -397,6 +406,7 @@ PL_EXPORT(int)		PL_is_functor(term_t t, functor_t f);
 PL_EXPORT(int)		PL_is_list(term_t t);
 PL_EXPORT(int)		PL_is_atomic(term_t t);
 PL_EXPORT(int)		PL_is_number(term_t t);
+PL_EXPORT(int)		PL_is_acyclic(term_t t);
 
 			/* Assign to term-references */
 PL_EXPORT(void)		PL_put_variable(term_t t);
@@ -664,6 +674,9 @@ UNICODE file functions.
 #define REP_FN		REP_MB
 #endif
 
+#define PL_DIFF_LIST	0x20000		/* PL_unify_chars() */
+
+
 #ifdef SIO_MAGIC			/* defined from <SWI-Stream.h> */
 		 /*******************************
 		 *	  STREAM SUPPORT	*
@@ -820,6 +833,7 @@ PL_EXPORT(int)  PL_handle_signals(void);
 #define PL_ACTION_FLUSH		9	/* Flush Prolog i/o buffer */
 #define PL_ACTION_GUIAPP	10	/* Win32: set when this is a gui */
 #define PL_ACTION_ATTACH_CONSOLE 11	/* MT: Attach a console */
+#define PL_GMP_SET_ALLOC_FUNCTIONS 12	/* GMP: do not change allocation functions */
 
 PL_EXPORT(int)	 PL_action(int, ...);	/* perform some action */
 PL_EXPORT(void) PL_on_halt(void (*)(int, void *), void *);
