@@ -1,4 +1,4 @@
-/*  $Id: pl-main.c,v 1.172 2007/05/11 09:20:43 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
@@ -400,6 +400,12 @@ but these options are  too  widely  used   as  they  are  to change them
 overnight. Returns -1 on error.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
+static int
+isoption(const char *av, const char *opt)
+{ return (streq(av, opt) || (av[0] == '-' && streq(av+1, opt)));
+}
+
+
 static int
 parseCommandLineOptions(int argc0, char **argv, int *compile)
 { int argc = argc0;
@@ -411,8 +417,6 @@ parseCommandLineOptions(int argc0, char **argv, int *compile)
 
     if ( streq(s, "-" ) )		/* pl <plargs> -- <app-args> */
       break;
-    if ( *s == '-' )
-      continue;				/* don't handle --intptr_t=value */
 
     if ( streq(s, "tty") )	/* +/-tty */
     { if ( s[-1] == '+' )
@@ -421,14 +425,20 @@ parseCommandLineOptions(int argc0, char **argv, int *compile)
 	clearFeatureMask(TTY_CONTROL_FEATURE);
 
       continue;
-    } else if ( streq(s, "nosignals") )
+    } else if ( isoption(s, "nosignals") )
     { clearFeatureMask(SIGNALS_FEATURE);
       continue;
-    } else if ( streq(s, "nodebug") )
+    } else if ( isoption(s, "nodebug") )
     { clearFeatureMask(DEBUGINFO_FEATURE);
       continue;
+    } else if ( streq(s, "-quiet") )
+    { GD->options.silent = TRUE;
+      continue;
     }
 
+    if ( *s == '-' )
+      continue;				/* don't handle --long=value */
+
     while(*s)
     { switch(*s)
       { case 'd':	if (argc > 1)
@@ -622,7 +632,7 @@ the script file. This  mode  is   selected  if  SCRIPT_BREAKDOWN_ARGS is
 defined by configure.
 
 On Windows this is simply passed as below.   We have to analyse the file
-ourselves. Unfortunately this needs to be done  on C as it might contain
+ourselves. Unfortunately this needs to be done  in C as it might contain
 stack-parameters.
 
 	{plwin.exe <file>}
@@ -700,17 +710,6 @@ script_argv(int argc, char **argv)
     char *av[MAXARGV];
     int  an = 0;
 
-#ifdef __WINDOWS__
-    if ( argc == 2 )
-    { char tmp[MAXPATHLEN];
-      char dir[MAXPATHLEN];
-
-      _xos_canonical_filename(argv[1], tmp, MAXPATHLEN, 0);
-      if ( IsAbsolutePath(tmp) )
-	chdir(DirName(tmp, dir));
-    }
-#endif
-
     fgets(buf, sizeof(buf), fd);
     if ( !strprefix(buf, "#!") )
     { fclose(fd);
@@ -964,10 +963,10 @@ static int
 usage()
 { static const cline lines[] = {
     "%s: Usage:\n",
-    "    1) %s -help      Display this message\n",
-    "    2) %s -v         Display version information\n",
-    "    3) %s -arch      Display architecture\n",
-    "    4) %s -dump-runtime-variables[=format]\n"
+    "    1) %s --help     Display this message (also -h)\n",
+    "    2) %s --version  Display version information (also -v)\n",
+    "    3) %s --arch     Display architecture\n",
+    "    4) %s --dump-runtime-variables[=format]\n"
     "                     Dump link info in sh(1) format\n",
     "    5) %s [options]\n",
     "    6) %s [options] [-o output] -c file ...\n",
@@ -981,10 +980,10 @@ usage()
     "    -F file          System initialisation file\n",
     "    -s file          Script source file\n",
     "    [+/-]tty         Allow tty control\n",
-    "    -nosignals       Do not modify any signal handling\n",
-    "    -nodebug         Omit generation of debug info\n",
     "    -O               Optimised compilation\n",
-    "    -q               Quiet operation\n",
+    "    --nosignals      Do not modify any signal handling\n",
+    "    --nodebug        Omit generation of debug info\n",
+    "    --quiet          Quiet operation (also -q)\n",
     NULL
   };
   const cline *lp = lines;
@@ -1091,12 +1090,19 @@ runtime_vars(int format)
 
 static int
 giveVersionInfo(const char *a)
-{ if ( streq(a, "-help") )
+{ if ( *a != '-' )
+    return FALSE;
+
+  if ( streq(a, "-help") || streq(a, "--help") || streq(a, "-h") )
     return usage();
-  if ( streq(a, "-arch") )
+  if ( streq(a, "-arch") || streq(a, "--arch") )
     return arch();
-  if ( streq(a, "-v") )
+  if ( streq(a, "--version") || streq(a, "-v") )
     return version();
+
+  if ( a[1] == '-' )			/* allow for --name versions */
+    a++;
+
   if ( streq(a, "-dump-runtime-variables") )
     return runtime_vars(FMT_SH);
   if ( streq(a, "-dump-runtime-variables=sh") )
@@ -1211,7 +1217,7 @@ PL_cleanup(int rval)
   }
 
   cleanupSignals();
-  freeStacks(LD);
+  freeStacks(PASS_LD1);
 #ifdef HAVE_DMALLOC_H
   dmalloc_verify(0);
 #endif
@@ -1220,7 +1226,6 @@ PL_cleanup(int rval)
   cleanupAtoms();
   cleanupFunctors();
   cleanupArith();
-  cleanupMemAlloc();
   cleanupInitialiseHooks();
   cleanupExtensions();
   cleanupOs();
@@ -1228,6 +1233,8 @@ PL_cleanup(int rval)
 #ifdef O_PLMT
   cleanupThreads();
 #endif
+  cleanupForeign();
+  cleanupMemAlloc();
 
   UNLOCK();				/* requires GD->thread.enabled */
 
@@ -1310,7 +1317,7 @@ vsysError(const char *fm, va_list args)
 action:
   Sfprintf(Serror, "\nAction? "); Sflush(Soutput);
   ResetTty();
-  switch(getSingleChar(Sinput))
+  switch(getSingleChar(Sinput, FALSE))
   { case 'a':
       pl_abort(ABORT_FATAL);
       break;
