@@ -1,4 +1,4 @@
-/*  $Id: pl-wic.c,v 1.141 2007/07/09 13:21:18 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
@@ -658,7 +658,7 @@ loadXRc(int c, IOSTREAM *fd ARG_LD)
 	{ atom_t name   = loadXR(fd);
 	  word   time   = getLong(fd);
 	  const char *s = stringAtom(name);
-	  SourceFile sf = lookupSourceFile(qlfFixSourcePath(s));
+	  SourceFile sf = lookupSourceFile(qlfFixSourcePath(s), TRUE);
 
 	  if ( !sf->time )
 	  { sf->time   = time;
@@ -958,7 +958,7 @@ loadPredicate(IOSTREAM *fd, int skip ARG_LD)
   def = proc->definition;
   if ( !skip && currentSource )
   { if ( def->definition.clauses )
-      redefineProcedure(proc, currentSource);
+      redefineProcedure(proc, currentSource, DISCONTIGUOUS_STYLE);
     addProcedureSourceFile(currentSource, proc);
   }
   if ( def->references == 0 && !def->hash_info )
@@ -1180,7 +1180,7 @@ qlfLoadSource(IOSTREAM *fd)
   DEBUG(1, if ( !streq(stringAtom(fname), str) )
 	     Sdprintf("Replaced path %s --> %s\n", str, stringAtom(fname)));
 
-  currentSource = lookupSourceFile(fname);
+  currentSource = lookupSourceFile(fname, TRUE);
   currentSource->time = time;
   currentSource->system = issys;
   if ( GD->bootsession )		/* (**) */
@@ -1337,8 +1337,9 @@ Note that we keep track of the `current procedure' to keep  all  clauses
 of a predicate together.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static Table savedXRTable;		/* saved XR entries */
-static intptr_t  savedXRTableId;		/* next id */
+static Table	savedXRTable;		/* saved XR entries */
+static intptr_t	savedXRTableId;		/* next id */
+static intptr_t registered_atoms;	/* safety check */
 
 #define STR_NOLEN ((size_t)-1)
 
@@ -1473,6 +1474,40 @@ putLong(long v, IOSTREAM *fd)		/* always 4 bytes */
 }
 
 
+static void
+freeXRSymbol(Symbol s)
+{ word w = (word)s->name;
+
+  if ( w&0x1 )
+  { w &= ~0x1;
+    if ( isAtom(w) )
+    { registered_atoms--;
+      PL_unregister_atom(w);
+      DEBUG(5, Sdprintf("UNREG: %s\n", stringAtom(w)));
+    }
+  }
+}
+
+
+void
+initXR()
+{ currentProc		    = NULL;
+  currentSource		    = NULL;
+  savedXRTable		    = newHTable(256);
+  savedXRTable->free_symbol = freeXRSymbol;
+  savedXRTableId	    = 0;
+  registered_atoms	    = 0;
+}
+
+
+void
+destroyXR()
+{ destroyHTable(savedXRTable);
+  savedXRTable = NULL;
+  assert(registered_atoms==0);
+}
+
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 XR (External Reference)  table  handling.   The  table  contains  atoms,
 functors  and  various  types  of    pointers   (Module,  Procedure  and
@@ -1481,7 +1516,8 @@ conflict with pointers. We assume -as in  many other places in the code-
 that pointers are 4-byte aligned.
 
 savedXRConstant()  must  be  used  for    atom_t  and  functor_t,  while
-savedXRPointer must be used for the pointers.
+savedXRPointer  must  be  used  for   the    pointers.   The  value  for
+savedXRConstant() is or-ed with 0x1 to avoid conflict with pointers.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static int
@@ -1506,10 +1542,17 @@ savedXR(void *xr, IOSTREAM *fd)
 
 static inline int
 savedXRConstant(word w, IOSTREAM *fd)
-{ assert(tag(w) == TAG_ATOM);		/* Only functor_t and atom_t */
-  w |= 0x1;				/* Avoid conflict with pointer */
-  
-  return savedXR((void *)w, fd);
+{ int rc;
+
+  assert(tag(w) == TAG_ATOM);		/* Only functor_t and atom_t */
+
+  if ( !(rc=savedXR((void *)(w|0x1), fd)) && isAtom(w) )
+  { registered_atoms++;
+    DEBUG(5, Sdprintf("REG: %s\n", stringAtom(w)));
+    PL_register_atom(w);
+  }
+
+  return rc;
 }
 
 
@@ -1889,10 +1932,7 @@ writeWicHeader(IOSTREAM *fd)
   else
     putString("<no home>",  STR_NOLEN, fd);
 
-  currentProc    = (Procedure) NULL;
-  currentSource  = (SourceFile) NULL;
-  savedXRTable   = newHTable(256);
-  savedXRTableId = 0;
+  initXR();
 
   DEBUG(2, Sdprintf("Header complete ...\n"));
   succeed;
@@ -1906,8 +1946,7 @@ writeWicTrailer(IOSTREAM *fd)
 
   closeProcedureWic(fd);
   Sputc('X', fd);
-  destroyHTable(savedXRTable);
-  savedXRTable = NULL;
+  destroyXR();
   Sputc('T', fd);
 
   wicFd = NULL;
@@ -2140,11 +2179,7 @@ qlfOpen(atom_t name)
   putNum(sizeof(word)*8, wicFd);
 
   putString(absname, STR_NOLEN, wicFd);
-
-  currentProc    = (Procedure) NULL;
-  currentSource  = (SourceFile) NULL;
-  savedXRTable   = newHTable(256);
-  savedXRTableId = 0;
+  initXR();
   initSourceMarks();
 
   succeed;
@@ -2160,9 +2195,7 @@ qlfClose(ARG1_LD)
   Sclose(fd);
   wicFd = NULL;
   mkWicFile = NULL;
-
-  destroyHTable(savedXRTable);
-  savedXRTable = NULL;
+  destroyXR();
   
   succeed;
 }
@@ -2373,8 +2406,8 @@ pl_qlf_start_module(term_t name)
   { GET_LD
     Module m;
 
-    if ( !PL_get_module(name, &m) )
-      return warning("qlf_start_module/1: argument must be an atom");
+    if ( !PL_get_module_ex(name, &m) )
+      fail;
   
     return qlfStartModule(m, wicFd PASS_LD);
   }
@@ -2389,8 +2422,8 @@ pl_qlf_start_sub_module(term_t name)
   { GET_LD
     Module m;
 
-    if ( !PL_get_module(name, &m) )
-      return warning("qlf_start_sub_module/1: argument must be an atom");
+    if ( !PL_get_module_ex(name, &m) )
+      fail;
   
     return qlfStartSubModule(m, wicFd PASS_LD);
   }
@@ -2408,7 +2441,7 @@ pl_qlf_start_file(term_t name)
     if ( !PL_get_atom_ex(name, &a) )
       fail;
   
-    return qlfStartFile(lookupSourceFile(a), wicFd PASS_LD);
+    return qlfStartFile(lookupSourceFile(a, TRUE), wicFd PASS_LD);
   }
 
   succeed;
@@ -2520,17 +2553,17 @@ pl_add_directive_wic(term_t term)
 
 
 word
-pl_import_wic(term_t module, term_t head)
+pl_import_wic(term_t module, term_t pi)
 { if ( wicFd )
   { GET_LD
-    Module m;
-    functor_t f;
+    Module m = NULL;
+    functor_t fd;
 
     if ( !PL_get_module(module, &m) ||
-	 !PL_get_functor(head, &f) )
-      return warning("$import_wic/3: instantiation fault");
+	 !get_functor(pi, &fd, &m, 0, GF_PROCEDURE) )
+      fail;
 
-    return importWic(lookupProcedure(f, m), wicFd PASS_LD);
+    return importWic(lookupProcedure(fd, m), wicFd PASS_LD);
   }
 
   succeed;
@@ -2629,7 +2662,7 @@ compileFile(char *file)
     fail;
   DEBUG(2, Sdprintf("pl_start_consult()\n"));
   pl_start_consult(f);
-  qlfStartFile(lookupSourceFile(nf), wicFd PASS_LD);
+  qlfStartFile(lookupSourceFile(nf, TRUE), wicFd PASS_LD);
   
   for(;;)
   { fid_t	 cid = PL_open_foreign_frame();
@@ -2720,7 +2753,7 @@ qlfCleanup()
   if ( getstr_buffer )
   { free(getstr_buffer);
     getstr_buffer = NULL;
-    getstr_buffer_size = 512;
+    getstr_buffer_size = 0;
   }
 }
 
